/**
 * KeyboardState.java
 * 4/1/11
 */

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * @author faculty
 *
 */
public class KeyboardState implements KeyListener {
    /**  Up key. */
    public static final int UP = 0;

    /**  Down key. */
    public static final int DOWN = 1;

    /**  Left key. */
    public static final int LEFT = 2;

    /**  Right key. */
    public static final int RIGHT = 3;

    /**  Space key. */
    public static final int SPACE = 4;

    /** Number of keys in pressed array. */
    public static final int NUM_KEYS = 5;

    /** Which keys are currently pressed. */
    private boolean[] pressedKeys;

    /**
     * Constructor, initialize key pressed array.
     */
    public KeyboardState() {
        pressedKeys = new boolean[NUM_KEYS];
    }

    /**
     * Return true if the specified key is pressed, false otherwise.
     * 
     * @param key key to check if being pressed
     * @return if specified key is being pressed
     */
    public final boolean isPressed(final int key) {
        return key >= 0 && key < NUM_KEYS && pressedKeys[key];
    }

    /**
     * Update pressed keys when a key is pressed.
     * 
     * @param event key event
     */
    public final void keyPressed(final KeyEvent event) {
        int keyCode = event.getKeyCode();
        if (keyCode == KeyEvent.VK_UP) {
            pressedKeys[UP] = true;
        } else if (keyCode == KeyEvent.VK_DOWN) {
            pressedKeys[DOWN] = true;
        } else if (keyCode == KeyEvent.VK_LEFT) {
            pressedKeys[LEFT] = true;
        } else if (keyCode == KeyEvent.VK_RIGHT) {
            pressedKeys[RIGHT] = true;
        } else if (keyCode == KeyEvent.VK_SPACE) {
            pressedKeys[SPACE] = true;
        }
    }

    /**
     * Update pressed keys when key is released.
     * 
     * @param event key event
     */
    public final void keyReleased(final KeyEvent event) {
        int keyCode = event.getKeyCode();
        if (keyCode == KeyEvent.VK_UP) {
            pressedKeys[UP] = false;
        } else if (keyCode == KeyEvent.VK_DOWN) {
            pressedKeys[DOWN] = false;
        } else if (keyCode == KeyEvent.VK_LEFT) {
            pressedKeys[LEFT] = false;
        } else if (keyCode == KeyEvent.VK_RIGHT) {
            pressedKeys[RIGHT] = false;
        } else if (keyCode == KeyEvent.VK_SPACE) {
            pressedKeys[SPACE] = false;
        }
    }

    /**
     * Do nothing if key is typed.
     * 
     * @param event key event
     */
    public void keyTyped(final KeyEvent event) {
    }

    /**
     * Return a string representation of which keys are being pressed.
     * 
     * @return string of which keys are being pressed
     */
    public final String toString() {
        String string = "";
        if (pressedKeys[UP]) {
            string += "up ";
        } else if (pressedKeys[DOWN]) {
            string += "down ";
        } else if (pressedKeys[LEFT]) {
            string += "left ";
        } else if (pressedKeys[RIGHT]) {
            string += "right ";
        } else if (pressedKeys[SPACE]) {
            string += "space ";
        }
        return string;
    }

}
