/**
 * Level.java
 * 4/1/11
 */

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.Iterator;
import javax.swing.JPanel;

/**
 * Platform game level.
 * 
 * @author CPSC170A
 */
public class Level extends JPanel {

    /** Prevent eclipse warning. */
    private static final long serialVersionUID = 1L;

    /** Platform sprites. */
    private Platforms platforms;

    /** Player sprite. */
    private Player player;

    /** Keyboard state. */
    private KeyboardState keyboardState;

    /** Viewport location. */
    private Point2D.Double viewport;

    /**
     * Constructor, initializes all sprites.
     */
    public Level() {
        player = new Player(Settings.INIT_PLAYER_X, Settings.INIT_PLAYER_Y,
                Settings.PLAYER_WIDTH, Settings.PLAYER_HEIGHT);
        switch (Settings.STORAGE_MODE) {
            case LIST:
                platforms = new PlatformList(new File(
                        Settings.PLATFORM_FILE_NAME));
                break;
            case MAP:
                platforms = new PlatformMap(new File(
                        Settings.PLATFORM_FILE_NAME));
                break;
            default:
                break;
        }
        keyboardState = new KeyboardState();
        viewport = new Point2D.Double(0.0, 0.0);
        addKeyListener(keyboardState);
        setFocusable(true);
        setPreferredSize(new Dimension(Settings.PANEL_WIDTH,
                Settings.PANEL_HEIGHT));
    }

    /**
     * Draw the level.
     * 
     * @param g graphics object to draw to
     */
    public final void paintComponent(final Graphics g) {
        super.paintComponent(g);
        Sprite platform;
        Iterator<Sprite> iter = platforms.iterator(new Rectangle2D.Double(0.0,
                0.0, Settings.PANEL_WIDTH, Settings.PANEL_HEIGHT));
        while (iter.hasNext()) {
            platform = iter.next();
            platform.paint(g, viewport);
        }
        player.paint(g, viewport);
    }

    /**
     * Update the location of the viewport based on the player location and
     * velocity.
     */
    private void updateViewport() {
        double localX = player.x - viewport.x;
        double localY = player.y - viewport.y;
        double rightBound = Settings.RIGHT_BOUND;
        double leftBound = Settings.LEFT_BOUND;
        double topBound = Settings.TOP_BOUND;
        double bottomBound = Settings.BOTTOM_BOUND;
        if (player.deltaX > 0.0 && localX + player.width > rightBound) {
            viewport.x += ((localX + player.width) - rightBound);
        }
        if (player.deltaX < 0.0 && localX < leftBound) {
            viewport.x -= (leftBound - (localX + player.width));
        }
        if (player.deltaY < 0.0 && localY < topBound) {
            viewport.y -= (topBound - localY);
        }
        if (player.deltaY > 0.0 && (localY + player.height) > bottomBound) {
            viewport.y += ((localY + player.height) - bottomBound);
        }
    }

    /**
     * Update this level based on the amount of time that has elapsed since the
     * last update.
     * 
     * @param elapsedTime the amount of time that has elapsed since the last
     *            update
     */
    public final void update(final long elapsedTime) {
        player.update(elapsedTime, keyboardState, platforms);
        updateViewport();
        repaint();
    }

}
