/**
 * LevelEditor.java
 * 4/1/11
 */

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.Iterator;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * Draw a list of cards.
 * 
 * @author CPSC170A
 */
public class LevelEditor extends JPanel implements MouseMotionListener,
        ActionListener {

    /** Prevent eclipse warning. */
    private static final long serialVersionUID = 1L;

    /** Window dimensions. */
    private static final Dimension WIN_DIM = new Dimension(640, 480);

    /** Panel dimensions. */
    private static final Dimension PANEL_DIM = new Dimension(65536, 1024);

    /** Window object. */
    private JFrame frame = new JFrame("Level Editor - Count: 0");

    /** List of platforms. */
    private PlatformList platforms = new PlatformList();

    /**
     * Constructor, initializes window.
     * 
     * @throws IOException
     */
    public LevelEditor() {
        JScrollPane scrollPane = new JScrollPane(this);
        scrollPane.setPreferredSize(WIN_DIM);
        setPreferredSize(PANEL_DIM);
        addMouseMotionListener(this);
        initMenu();
        frame.getContentPane().add(scrollPane);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }

    /**
     * Initialize window menu bar.
     */
    public final void initMenu() {
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
        JMenuItem newMenuItem = new JMenuItem("New");
        JMenuItem openMenuItem = new JMenuItem("Open");
        JMenuItem saveMenuItem = new JMenuItem("Save");
        JMenuItem exitMenuItem = new JMenuItem("Exit");
        newMenuItem.addActionListener(this);
        openMenuItem.addActionListener(this);
        saveMenuItem.addActionListener(this);
        exitMenuItem.addActionListener(this);
        fileMenu.add(newMenuItem);
        fileMenu.add(openMenuItem);
        fileMenu.add(saveMenuItem);
        fileMenu.add(exitMenuItem);
        menuBar.add(fileMenu);
        frame.setJMenuBar(menuBar);
        frame.setVisible(true);
    }

    /**
     * Runs the application.
     * 
     * @param args command line argument
     */
    public static void main(final String[] args) {
        new LevelEditor();
    }

    /**
     * Draw the list of cards.
     * 
     * @param g graphics object to draw to
     */
    public final void paintComponent(final Graphics g) {
        super.paintComponent(g);
        Iterator<Sprite> iter = platforms.iterator(new Rectangle2D.Double(0, 0,
                0, 0));
        while (iter.hasNext()) {
            iter.next().paint(g, new Point2D.Double(0, 0));
        }
    }

    /**
     * Add a platform when the mouse is dragged.
     * 
     * @param event mouse event
     */
    public final void mouseDragged(final MouseEvent event) {
        platforms.getList().add(
                new Sprite(event.getX(), event.getY(), Settings.PLATFORM_SIZE,
                        Settings.PLATFORM_SIZE, 0, 0, null));
        frame.setTitle("Level Editor - Count: " + platforms.getList().size());
        repaint();
    }

    /**
     * Unused in this program.
     * 
     * @param event moust event
     */
    public void mouseMoved(final MouseEvent event) {
    }

    /**
     * Handle menu selection.
     * 
     * @param event action event
     */
    public final void actionPerformed(final ActionEvent event) {
        String menuText = ((JMenuItem) event.getSource()).getText();
        if (menuText.equals("New")) {
            platforms = new PlatformList();
            repaint();
        } else if (menuText.equals("Open")) {
            JFileChooser fileChooser = new JFileChooser();
            int response;
            fileChooser.setCurrentDirectory(new File("."));
            response = fileChooser.showOpenDialog(frame);
            if (response == JFileChooser.APPROVE_OPTION) {
                platforms = new PlatformList(fileChooser.getSelectedFile());
            }
            repaint();
        } else if (menuText.equals("Save")) {
            JFileChooser fileChooser = new JFileChooser();
            int response;
            fileChooser.setCurrentDirectory(new File("."));
            response = fileChooser.showSaveDialog(frame);
            if (response == JFileChooser.APPROVE_OPTION) {
                platforms.write(fileChooser.getSelectedFile());
            }
        } else if (menuText.equals("Exit")) {
            System.exit(0);
        }
    }
}
