/**
 * PlatformList.java
 * 4/1/11
 */

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Scanner;

/**
 * Represents a collection of platforms.
 * 
 * @author CPSC170A
 */
public class PlatformList extends Platforms {

    /** List of platform sprites. */
    private ArrayList<Sprite> platforms;

    /**
     * Constructor, Initializes the platform list from the specified file.
     * 
     * @param file file containing platforms
     */
    public PlatformList(final File file) {
        this();
        read(file);
    }

    /**
     * constructor, initializes an empty platform list.
     */
    public PlatformList() {
        platforms = new ArrayList<Sprite>();
    }

    /**
     * Get list of platforms.
     * 
     * @return list of platforms
     */
    public final ArrayList<Sprite> getList() {
        return platforms;
    }

    /**
     * Add the platforms in the specified file to this platform list.
     * 
     * @param file file to read
     */
    private void read(final File file) {
        ArrayList<Sprite> readPlatforms = new ArrayList<Sprite>();
        try {
            Scanner scan = new Scanner(file);
            double x, y, width, height;
            while (scan.hasNext()) {
                x = scan.nextDouble();
                y = scan.nextDouble();
                width = scan.nextDouble();
                height = scan.nextDouble();
                readPlatforms.add(new Sprite(x, y, width, height, 0, 0, null));
            }
        } catch (FileNotFoundException e) {
            System.out.println("Unable to read file: " + file);
            readPlatforms = new ArrayList<Sprite>();
        }
        if (readPlatforms.size() > 0) {
            platforms.addList(readPlatforms);
        }
    }

    /**
     * Write the platforms in this platform list to the specified file.
     * 
     * @param file file to write to
     */
    public final void write(final File file) {
        PrintWriter out = null;
        try {
            out = new PrintWriter(file);
            for (Sprite platform : platforms) {
                out.println(platform.x + " " + platform.y + " "
                        + platform.width + " " + platform.height);
            }
        } catch (IOException e) {
            System.out.println("Unable to write file: " + file);
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }

    /**
     * Returns an iterator over the specified rectangle for this platform list.
     * 
     * @param rect rectangle to iterate over
     * @return an iterator for this platform list.
     */
    public final Iterator<Sprite> iterator(final Rectangle2D.Double rect) {
        return platforms.iterator();
    }

}
