/**
 * PlatformMap.java
 * 4/1/11
 */

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Iterator;
import java.util.Scanner;

/**
 * @author CPSC170
 * 
 */
public class PlatformMap extends Platforms {
    /** List of platform sprites. */
    private Map2D<Double, Sprite> platforms;

    /**
     * Constructor, Initializes the platform list from the specified file.
     * 
     * @param file file containing platforms
     */
    public PlatformMap(final File file) {
        this();
        read(file);
    }

    /**
     * constructor, initializes an empty platform list.
     */
    public PlatformMap() {
        platforms = new Map2D<Double, Sprite>();
    }

    /**
     * Add the platforms in the specified file to this platform list.
     * 
     * @param file file to read
     */
    private void read(final File file) {
        Map2D<Double, Sprite> readPlatforms = new Map2D<Double, Sprite>();
        try {
            Scanner scan = new Scanner(file);
            double x, y, width, height;
            while (scan.hasNext()) {
                x = scan.nextDouble();
                y = scan.nextDouble();
                width = scan.nextDouble();
                height = scan.nextDouble();
                readPlatforms.put(x, y, new Sprite(x, y, width, height, 0, 0,
                        null));
            }
        } catch (FileNotFoundException e) {
            System.out.println("Unable to read file: " + file);
            readPlatforms = null;
        }
        if (readPlatforms != null) {
            platforms = readPlatforms;
        }
    }

    /**
     * Returns an iterator over the specified rectangle for this platform list.
     * 
     * @param rect rectangle to iterate over
     * @return an iterator for this platform list.
     */
    public final Iterator<Sprite> iterator(final Rectangle2D.Double rect) {
        return platforms.iterator(rect.x, rect.y, rect.x + rect.width, rect.y
                + rect.height);
    }

}
