/**
 * Platformer.java
 * 4/1/11
 */

import javax.swing.JFrame;

/**
 * A simple platform game.
 * 
 * @author CPSC170A
 */
public class Platformer implements Runnable {

    /** Number of milliseconds in a second. */
    private static final long MILLI_IN_SEC = 1000;

    /** System time of last frame. */
    private long previousTime;

    /** Number of frames elapsed this second. */
    private int frameCnt = 0;

    /** Number of milliseconds elapsed this second. */
    private long timeCnt = 0;

    /** Program window. */
    private JFrame frame;

    /** Level to draw in window. */
    private Level level;

    /**
     * Runs the application.
     * 
     * @param args command line argument
     */
    public static void main(final String[] args) {
        new Platformer();
    }

    /**
     * Constructor, initializes this platform game.
     */
    public Platformer() {
        Thread thread = new Thread(this);
        frame = new JFrame("Platformer - ???fps");
        level = new Level();
        previousTime = System.currentTimeMillis();
        frame.getContentPane().add(level);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
        thread.start();
    }

    /**
     * Run loop for the game.
     */
    public final void run() {
        long currentTime;
        long elapsedTime;
        boolean running = true;
        while (running) {
            currentTime = System.currentTimeMillis();
            elapsedTime = currentTime - previousTime;
            previousTime = currentTime;
            timeCnt += elapsedTime;
            frameCnt++;
            if (timeCnt > MILLI_IN_SEC) {
                frame.setTitle("Platformer - " + frameCnt + "fps");
                frameCnt = 0;
                timeCnt = 0;
            }
            level.update(elapsedTime);
            try {
                Thread.sleep(1);
            } catch (Exception e) {
                System.out.println("Error sleeping.");
                running = false;
            }
        }
    }

}
