/**
 * Player.java
 * 4/1/11
 */

import java.awt.geom.Rectangle2D;
import java.util.Iterator;

/**
 * Represents the player in a simple platform game.
 * 
 * @author CPSC170A
 * 
 */
public class Player extends Sprite {

    /** Whether player is on a platform. */
    private boolean onPlatform;

    /**
     * Constructor, initializes a new ball.
     * 
     * @param initX initial x location
     * @param initY initial y location
     * @param initWidth initial width
     * @param initHeight initial height
     */
    public Player(final double initX, final double initY,
            final double initWidth, final double initHeight) {
        super(initX, initY, initWidth, initHeight, 0.0, 0.0, null);
        onPlatform = false;
    }

    /**
     * Update the location, velocity, and image of the player.
     * 
     * @param elapsedTime time elapsed
     * @param keyboardState current game keyboard state
     * @param platforms list of platforms
     */
    public final void update(final long elapsedTime,
            final KeyboardState keyboardState, final Platforms platforms) {
        updateVelocity(keyboardState);
        updateLocation(elapsedTime, platforms);
    }

    /**
     * Update the velocity of the player.
     * 
     * @param keyboardState current game keyboard state
     */
    private void updateVelocity(final KeyboardState keyboardState) {
        if (keyboardState.isPressed(KeyboardState.LEFT)) {
            deltaX = Settings.INIT_PLAYER_VEL * -1.0;
        } else if (keyboardState.isPressed(KeyboardState.RIGHT)) {
            deltaX = Settings.INIT_PLAYER_VEL;
        } else {
            deltaX = 0.0;
        }
        if (keyboardState.isPressed(KeyboardState.UP) && onPlatform) {
            deltaY -= Settings.JUMP;
        }
        deltaY += Settings.GRAVITY;
    }

    /**
     * Update the location of this player.
     * 
     * @param elapsedTime time elapsed
     * @param platforms list of platforms
     */
    private void updateLocation(final long elapsedTime,
            final Platforms platforms) {
        Sprite platform;
        double topOverlap, bottomOverlap, leftOverlap, rightOverlap;
        x += deltaX * elapsedTime;
        y += deltaY * elapsedTime;
        onPlatform = false;
        Iterator<Sprite> iter = platforms.iterator(new Rectangle2D.Double(x
                - Settings.PLATFORM_SIZE, y - Settings.PLATFORM_SIZE, x + width
                + Settings.PLATFORM_SIZE, y + height + Settings.PLATFORM_SIZE));
        while (iter.hasNext()) {
            platform = iter.next();
            if (intersect(platform)) {
                topOverlap = topOverlap(platform);
                bottomOverlap = bottomOverlap(platform);
                leftOverlap = leftOverlap(platform);
                rightOverlap = rightOverlap(platform);
                if (topOverlap < bottomOverlap && topOverlap < leftOverlap
                        && topOverlap < rightOverlap && deltaY > 0.0) {
                    y = platform.y - height;
                    deltaY = 0.0;
                    onPlatform = true;
                } else if (bottomOverlap < topOverlap
                        && bottomOverlap < leftOverlap
                        && bottomOverlap < rightOverlap && deltaY < 0.0) {
                    y = platform.y + platform.height;
                    deltaY = 0.0;
                } else if (leftOverlap < topOverlap
                        && leftOverlap < bottomOverlap
                        && leftOverlap < rightOverlap && deltaX > 0.0) {
                    x = platform.x - width;
                    deltaX = 0.0;
                } else if (rightOverlap < topOverlap
                        && rightOverlap < bottomOverlap
                        && rightOverlap < leftOverlap && deltaX < 0.0) {
                    x = platform.x + platform.width;
                    deltaX = 0.0;
                }
            }
        }
    }

}
